
From: Bodo Stroesser <bstroesser@fujitsu-siemens.com>

Checksum handling largely depends on the subarch.

Thus, I renamed i386 arch_csum_partial in arch/um/sys-i386/checksum.S back to
csum_partial, removed csum_partial from arch/um/kernel/checksum.c and shifted
EXPORT_SYMBOL(csum_partial) to arch/um/sys-i386/ksyms.c.

Then, csum_partial_copy_to and csum_partial_copy_from were shifted from
arch/um/kernel/checksum.c to arch/um/include/sysdep-i386/checksum.h and
inserted in the calling functions csum_partial_copy_from_user() and
csum_and_copy_to_user().

Now, arch/um/kernel/checksum.c is empty and removed.

Signed-off-by: Bodo Stroesser <bstroesser@fujitsu-siemens.com>
Signed-off-by: Jeff Dike <jdike@addtoit.com>
Signed-off-by: Andrew Morton <akpm@osdl.org>
---

 arch/um/include/sysdep-i386/checksum.h |   39 ++++++++++++++++++---------------
 arch/um/kernel/Makefile                |    2 -
 arch/um/kernel/checksum.c              |   36 ------------------------------
 arch/um/sys-i386/checksum.S            |    6 ++---
 arch/um/sys-i386/ksyms.c               |    3 --
 5 files changed, 27 insertions(+), 59 deletions(-)

diff -puN arch/um/include/sysdep-i386/checksum.h~uml-s390-preparation-checksumming-done-in-arch-code arch/um/include/sysdep-i386/checksum.h
--- 25/arch/um/include/sysdep-i386/checksum.h~uml-s390-preparation-checksumming-done-in-arch-code	2005-05-03 16:14:15.000000000 -0700
+++ 25-akpm/arch/um/include/sysdep-i386/checksum.h	2005-05-03 16:14:15.000000000 -0700
@@ -24,19 +24,6 @@ unsigned int csum_partial(const unsigned
 			  unsigned int sum);
 
 /*
- * the same as csum_partial, but copies from src while it
- * checksums, and handles user-space pointer exceptions correctly, when needed.
- *
- * here even more important to align src and dst on a 32-bit (or even
- * better 64-bit) boundary
- */
-
-unsigned int csum_partial_copy_to(const unsigned char *src, unsigned char *dst,
-				  int len, int sum, int *err_ptr);
-unsigned int csum_partial_copy_from(const unsigned char *src, unsigned char *dst,
-				    int len, int sum, int *err_ptr);
-
-/*
  *	Note: when you get a NULL pointer exception here this means someone
  *	passed in an incorrect kernel address to one of these functions.
  *
@@ -52,11 +39,24 @@ unsigned int csum_partial_copy_nocheck(c
 	return(csum_partial(dst, len, sum));
 }
 
+/*
+ * the same as csum_partial, but copies from src while it
+ * checksums, and handles user-space pointer exceptions correctly, when needed.
+ *
+ * here even more important to align src and dst on a 32-bit (or even
+ * better 64-bit) boundary
+ */
+
 static __inline__
 unsigned int csum_partial_copy_from_user(const unsigned char *src, unsigned char *dst,
 					 int len, int sum, int *err_ptr)
 {
-	return csum_partial_copy_from(src, dst, len, sum, err_ptr);
+	if(copy_from_user(dst, src, len)){
+		*err_ptr = -EFAULT;
+		return(-1);
+	}
+
+	return csum_partial(dst, len, sum);
 }
 
 /*
@@ -67,7 +67,6 @@ unsigned int csum_partial_copy_from_user
  */
 
 #define csum_partial_copy_fromuser csum_partial_copy_from_user
-unsigned int csum_partial_copy(const unsigned char *src, unsigned char *dst, int len, int sum);
 
 /*
  *	This is a version of ip_compute_csum() optimized for IP headers,
@@ -196,8 +195,14 @@ static __inline__ unsigned int csum_and_
 						     unsigned char *dst,
 						     int len, int sum, int *err_ptr)
 {
-	if (access_ok(VERIFY_WRITE, dst, len))
-		return(csum_partial_copy_to(src, dst, len, sum, err_ptr));
+	if (access_ok(VERIFY_WRITE, dst, len)){
+		if(copy_to_user(dst, src, len)){
+			*err_ptr = -EFAULT;
+			return(-1);
+		}
+
+		return csum_partial(src, len, sum);
+	}
 
 	if (len)
 		*err_ptr = -EFAULT;
diff -puN arch/um/kernel/checksum.c~uml-s390-preparation-checksumming-done-in-arch-code arch/um/kernel/checksum.c
--- 25/arch/um/kernel/checksum.c~uml-s390-preparation-checksumming-done-in-arch-code	2005-05-03 16:14:15.000000000 -0700
+++ 25-akpm/arch/um/kernel/checksum.c	2005-05-03 16:14:15.000000000 -0700
@@ -1,36 +0,0 @@
-#include "asm/uaccess.h"
-#include "linux/errno.h"
-#include "linux/module.h"
-
-unsigned int arch_csum_partial(const unsigned char *buff, int len, int sum);
-
-unsigned int csum_partial(unsigned char *buff, int len, int sum)
-{
-        return arch_csum_partial(buff, len, sum);
-}
-
-EXPORT_SYMBOL(csum_partial);
-
-unsigned int csum_partial_copy_to(const unsigned char *src,
-                                  unsigned char __user *dst, int len, int sum,
-                                  int *err_ptr)
-{
-        if(copy_to_user(dst, src, len)){
-                *err_ptr = -EFAULT;
-                return(-1);
-        }
-
-        return(arch_csum_partial(src, len, sum));
-}
-
-unsigned int csum_partial_copy_from(const unsigned char __user *src,
-                                    unsigned char *dst,	int len, int sum,
-                                    int *err_ptr)
-{
-        if(copy_from_user(dst, src, len)){
-                *err_ptr = -EFAULT;
-                return(-1);
-        }
-
-        return arch_csum_partial(dst, len, sum);
-}
diff -puN arch/um/kernel/Makefile~uml-s390-preparation-checksumming-done-in-arch-code arch/um/kernel/Makefile
--- 25/arch/um/kernel/Makefile~uml-s390-preparation-checksumming-done-in-arch-code	2005-05-03 16:14:15.000000000 -0700
+++ 25-akpm/arch/um/kernel/Makefile	2005-05-03 16:14:15.000000000 -0700
@@ -6,7 +6,7 @@
 extra-y := vmlinux.lds
 clean-files :=
 
-obj-y = checksum.o config.o exec_kern.o exitcode.o \
+obj-y = config.o exec_kern.o exitcode.o \
 	helper.o init_task.o irq.o irq_user.o ksyms.o main.o mem.o mem_user.o \
 	physmem.o process.o process_kern.o ptrace.o reboot.o resource.o \
 	sigio_user.o sigio_kern.o signal_kern.o signal_user.o smp.o \
diff -puN arch/um/sys-i386/checksum.S~uml-s390-preparation-checksumming-done-in-arch-code arch/um/sys-i386/checksum.S
--- 25/arch/um/sys-i386/checksum.S~uml-s390-preparation-checksumming-done-in-arch-code	2005-05-03 16:14:15.000000000 -0700
+++ 25-akpm/arch/um/sys-i386/checksum.S	2005-05-03 16:14:15.000000000 -0700
@@ -38,7 +38,7 @@ unsigned int csum_partial(const unsigned
 		
 .text
 .align 4
-.globl arch_csum_partial								
+.globl csum_partial
 		
 #ifndef CONFIG_X86_USE_PPRO_CHECKSUM
 
@@ -49,7 +49,7 @@ unsigned int csum_partial(const unsigned
 	   * Fortunately, it is easy to convert 2-byte alignment to 4-byte
 	   * alignment for the unrolled loop.
 	   */		
-arch_csum_partial:	
+csum_partial:
 	pushl %esi
 	pushl %ebx
 	movl 20(%esp),%eax	# Function arg: unsigned int sum
@@ -119,7 +119,7 @@ arch_csum_partial:	
 
 /* Version for PentiumII/PPro */
 
-arch_csum_partial:
+csum_partial:
 	pushl %esi
 	pushl %ebx
 	movl 20(%esp),%eax	# Function arg: unsigned int sum
diff -puN arch/um/sys-i386/ksyms.c~uml-s390-preparation-checksumming-done-in-arch-code arch/um/sys-i386/ksyms.c
--- 25/arch/um/sys-i386/ksyms.c~uml-s390-preparation-checksumming-done-in-arch-code	2005-05-03 16:14:15.000000000 -0700
+++ 25-akpm/arch/um/sys-i386/ksyms.c	2005-05-03 16:14:15.000000000 -0700
@@ -13,5 +13,4 @@ EXPORT_SYMBOL(__down_failed_trylock);
 EXPORT_SYMBOL(__up_wakeup);
 
 /* Networking helper routines. */
-EXPORT_SYMBOL(csum_partial_copy_from);
-EXPORT_SYMBOL(csum_partial_copy_to);
+EXPORT_SYMBOL(csum_partial);
_
